﻿// ===============================================================================
//  解决方案：置换型模板引擎
//  项目名称：ReplacementTemplateEngine
//  文件名称：FormatableVariableLabel.cs
//  代码作者：网鸟老兵团(Me.YMind)
//  创建时间：2012-04-03 22:55
// ===============================================================================
//  Copyright © Htmlbird.Net. All rights reserved .
//  官方网站：http://www.htmlbird.net/
//  技术论坛：http://bbs.htmlbird.net/
//  版权所有：网鸟IT技术论坛 彦铭工作室
// ===============================================================================

using System;
using System.Collections.Generic;

namespace ReplacementTemplateEngine
{
	/// <summary>
	/// 表示可自定义格式化的变量标签。
	/// </summary>
	public sealed class FormattableVariableLabel : VariableLabel
	{
		/// <summary>
		/// 初始化 <see cref="FormattableVariableLabel"/> 的新实例。
		/// </summary>
		/// <param name="line">行数。</param>
		/// <param name="column">列数。</param>
		/// <param name="name">变量标记的名称。</param>
		/// <param name="format">变量标记的可选自定义格式化字符串。</param>
		public FormattableVariableLabel(int line, int column, string name, string format) : base(line, column, name)
		{
			if (format == null) throw new ArgumentNullException("format");
			if (format.Length == 0) throw new ArgumentOutOfRangeException("format");

			this.Format = format;
		}

		/// <summary>
		/// 获取变量标记的可选自定义格式化字符串。
		/// </summary>
		public string Format { get; private set; }

		/// <summary>
		/// 处置当前元素。
		/// </summary>
		/// <param name="variables">与当前元素关联的对象。</param>
		/// <returns>返回 <see cref="System.String"/>。</returns>
		public override string Process(Dictionary<string, object> variables)
		{
			if (variables == null || variables.Count == 0) return String.Empty;
			if (variables.ContainsKey(this.Name) == false) return String.Empty;

			var obj = variables[this.Name] as IFormattable;

			return obj == null ? variables[this.Name].ToString() : obj.ToString(this.Format, null);
		}
	}
}
