﻿// ===============================================================================
//  解决方案：置换型模板引擎
//  项目名称：ReplacementTemplateEngine
//  文件名称：TemplateParser.cs
//  代码作者：网鸟老兵团(Me.YMind)
//  创建时间：2012-04-04 12:35
// ===============================================================================
//  Copyright © Htmlbird.Net. All rights reserved .
//  官方网站：http://www.htmlbird.net/
//  技术论坛：http://bbs.htmlbird.net/
//  版权所有：网鸟IT技术论坛 彦铭工作室
// ===============================================================================

using System;
using System.Collections.Generic;

namespace ReplacementTemplateEngine.Parser
{
	/// <summary>
	/// 表示模板解析器。
	/// </summary>
	internal sealed class TemplateParser : ITemplateParser
	{
		private readonly TemplateLexer _lexer;

		/// <summary>
		/// 初始化 <see cref="TemplateParser"/> 类的新实例。
		/// </summary>
		/// <param name="lexer">指定 <see cref="TemplateLexer"/> 对象的实例。</param>
		public TemplateParser(TemplateLexer lexer)
		{
			if (lexer == null) throw new ArgumentNullException("lexer");

			this._lexer = lexer;
		}

		#region ITemplateParser Members

		/// <summary>
		/// 解析模板代码。
		/// </summary>
		/// <returns>返回 <see cref="Element"/> 对象的集合。</returns>
		public List<Element> Parse()
		{
			var elements = new List<Element>();
			var token = this._lexer.Next();

			do
			{
				Element element;

				switch (token.Kind)
				{
					case TokenKind.Text:
						element = new TextElement(token.Line, token.Column, token.Text);
						elements.Add(element);
						token = this._lexer.Next();
						break;

					case TokenKind.Label:
						element = new VariableLabel(token.Line, token.Column, token.Text);

						var nextToken = this._lexer.Next();

						if (nextToken.Kind == TokenKind.FormatStringPreamble)
						{
							nextToken = this._lexer.Next();

							if (nextToken.Kind == TokenKind.FormatString)
							{
								element = new FormattableVariableLabel(token.Line, token.Column, token.Text, nextToken.Text);
								elements.Add(element);
								token = this._lexer.Next();
							}
							else
							{
								elements.Add(element);
								token = nextToken;

								continue;
							}
						}
						else
						{
							elements.Add(element);
							token = nextToken;

							continue;
						}
						break;

					case TokenKind.LeftBracket:
					case TokenKind.RightBracket:
					case TokenKind.FormatStringPreamble:
						token = this._lexer.Next();
						break;
				}
			}
			while (token.Kind != TokenKind.EOF);

			return elements;
		}

		#endregion
	}
}
