﻿// ===============================================================================
//  解决方案：置换型模板引擎
//  项目名称：ReplacementTemplateEngine
//  文件名称：Template.cs
//  代码作者：网鸟老兵团(Me.YMind)
//  创建时间：2012-04-03 22:28
// ===============================================================================
//  Copyright © Htmlbird.Net. All rights reserved .
//  官方网站：http://www.htmlbird.net/
//  技术论坛：http://bbs.htmlbird.net/
//  版权所有：网鸟IT技术论坛 彦铭工作室
// ===============================================================================

using System;
using System.Collections.Generic;
using System.IO;
using ReplacementTemplateEngine.Parser;

namespace ReplacementTemplateEngine
{
	/// <summary>
	/// 表示一个模板。
	/// </summary>
	public sealed class Template : ITemplate
	{
		/// <summary>
		/// 初始化 <see cref="Template"/> 类的新实例。
		/// </summary>
		/// <param name="elements">模板元素的集合。</param>
		public Template(List<Element> elements)
		{
			if (elements == null) throw new ArgumentNullException("elements");
			if (elements.Count == 0) throw new ArgumentOutOfRangeException("elements");

			this.Elements = elements;
		}

		#region ITemplate Members

		/// <summary>
		/// 获取模板的标签库。
		/// </summary>
		public List<Element> Elements { get; private set; }

		#endregion

		/// <summary>
		/// 从模板文件初始化模板对象。
		/// </summary>
		/// <param name="filename">包含模板文件完整路径的字符串。</param>
		/// <returns>返回 <see cref="Template"/>。</returns>
		public static Template FromFile(string filename)
		{
			if (filename == null) throw new ArgumentNullException("filename");
			if (filename.Length == 0) throw new ArgumentOutOfRangeException("filename");

			if (File.Exists(filename) == false) throw new FileNotFoundException("指定的模板文档不存在！", filename);

			using (var reader = new StreamReader(filename))
			{
				var data = reader.ReadToEnd();

				return FromString(data);
			}
		}

		/// <summary>
		/// 从模板代码初始化模板对象。
		/// </summary>
		/// <param name="templateString">包含模板代码的字符串。</param>
		/// <returns>返回 <see cref="Template"/>。</returns>
		public static Template FromString(string templateString)
		{
			if (templateString == null) throw new ArgumentNullException("templateString");
			if (templateString.Length == 0) throw new ArgumentOutOfRangeException("templateString");

			var lexer = new TemplateLexer(templateString);
			var parser = new TemplateParser(lexer);
			var elements = parser.Parse();

			return new Template(elements);
		}
	}
}
