﻿// ===============================================================================
//  解决方案：置换型模板引擎
//  项目名称：ReplacementTemplateEngine
//  文件名称：TemplateEngine.cs
//  代码作者：网鸟老兵团(Me.YMind)
//  创建时间：2012-04-03 21:50
// ===============================================================================
//  Copyright © Htmlbird.Net. All rights reserved .
//  官方网站：http://www.htmlbird.net/
//  技术论坛：http://bbs.htmlbird.net/
//  版权所有：网鸟IT技术论坛 彦铭工作室
// ===============================================================================

using System;
using System.Collections.Generic;
using System.IO;

namespace ReplacementTemplateEngine
{
	/// <summary>
	/// 表示模板引擎。
	/// </summary>
	public sealed class TemplateEngine : ITemplateEngine
	{
		private readonly Dictionary<string, object> _variables = new Dictionary<string, object>();
		private TextWriter _writer;

		/// <summary>
		/// 初始化 <see cref="TemplateEngine"/> 类的新实例。
		/// </summary>
		/// <param name="template">指定一个模板实例。</param>
		public TemplateEngine(Template template)
		{
			if (template == null) throw new ArgumentNullException("template");

			this.Template = template;
		}

		#region ITemplateEngine Members

		/// <summary>
		/// 获取模板。
		/// </summary>
		public Template Template { get; private set; }

		/// <summary>
		/// 设定变量标记的置换值。
		/// </summary>
		/// <param name="key">键名。</param>
		/// <param name="value">值。</param>
		public void SetVariable(string key, object value)
		{
			// 在这里我们暂时不考虑多线程问题
			this._variables[key] = value;
		}

		/// <summary>
		/// 清空变量标记的置换值。
		/// </summary>
		public void ClearVariables() { this._variables.Clear(); }

		/// <summary>
		/// 处理模板。将处理结果保存到字符编写器中。
		/// </summary>
		/// <param name="writer">指定一个字符编写器。</param>
		public void Process(TextWriter writer)
		{
			this._writer = writer;

			foreach (var element in this.Template.Elements)
			{
				var textElement = element as TextElement;

				if (textElement == null)
				{
					var variableLabel = element as VariableLabel;

					if (variableLabel == null)
					{
						this._writer.Write("[");
						this._writer.Write(element.ToString());
						this._writer.Write("]");
					}
					else this._writer.Write(variableLabel.Process(this._variables));
				}
				else this._writer.Write(textElement.Text);
			}
		}

		/// <summary>
		/// 处理模板。并将结果作为字符串返回。
		/// </summary>
		/// <returns>返回 <see cref="System.String"/>。</returns>
		public string Process()
		{
			var writer = new StringWriter();

			this.Process(writer);

			return writer.ToString();
		}


		/// <summary>
		/// 删除变量标记的置换值。
		/// </summary>
		/// <param name="key">键名。</param>
		public void RemoveVariable(string key)
		{
			if (this._variables.ContainsKey(key) == false) return;

			this._variables.Remove(key);
		}

		#endregion

		/// <summary>
		/// 从模板文件初始化 <see cref="TemplateEngine"/> 类的新实例。
		/// </summary>
		/// <param name="filename">包含模板文件完整路径的字符串。</param>
		/// <returns>返回 <see cref="TemplateEngine"/>。</returns>
		public static TemplateEngine FromFile(string filename)
		{
			var template = Template.FromFile(filename);

			return new TemplateEngine(template);
		}

		/// <summary>
		/// 从模板代码初始化 <see cref="TemplateEngine"/> 类的新实例。
		/// </summary>
		/// <param name="templateString">包含模板代码的字符串。</param>
		/// <returns>返回 <see cref="TemplateEngine"/>。</returns>
		public static TemplateEngine FromString(string templateString)
		{
			var template = Template.FromString(templateString);

			return new TemplateEngine(template);
		}
	}
}
