﻿// ===============================================================================
//  解决方案：置换型模板引擎
//  项目名称：ReplacementTemplateEngine
//  文件名称：TemplateEngineUnitTests.cs
//  代码作者：网鸟老兵团(Me.YMind)
//  创建时间：2012-04-03 23:31
// ===============================================================================
//  Copyright © Htmlbird.Net. All rights reserved .
//  官方网站：http://www.htmlbird.net/
//  技术论坛：http://bbs.htmlbird.net/
//  版权所有：网鸟IT技术论坛 彦铭工作室
// ===============================================================================

using System;
using System.Diagnostics;
using NUnit.Framework;
using ReplacementTemplateEngine.Parser;

namespace ReplacementTemplateEngine
{
	[TestFixture]
	public sealed class TemplateEngineUnitTests
	{
		private const string _templateString = "[<time>{CreationTime:yyyy年MM月dd日 HH:mm:ss}</time>]\r\n<a href=\"{url}\">{title}</a>";
		private const string _html = "[<time>2012年04月03日 16:30:24</time>]\r\n<a href=\"http://www.ymind.net/\">陈彦铭的博客</a>";

		[Test]
		public void ProcessTest()
		{
			var templateEngine = TemplateEngine.FromString(_templateString);

			templateEngine.SetVariable("url", "http://www.ymind.net/");
			templateEngine.SetVariable("title", "陈彦铭的博客");
			templateEngine.SetVariable("CreationTime", new DateTime(2012, 4, 3, 16, 30, 24));

			var html = templateEngine.Process();

			Trace.WriteLine(html);

			// 还记得第一节课我就说，我为了简化代码架构使用了单元测试的方法来做的demo代码，那个不是真正的单元测试
			// 因为在那个时候，我们的代码中没有包含结果验证的过程

			// 对输出结果进行测试验证，首先不能是null
			Assert.NotNull(html);

			// 输出结果必须与预期结果完全一致
			Assert.AreEqual(_html, html);

			// 如果以上两个验证无法通过，那么执行的时候必定会报错！
		}

		[Test]
		public void TokensTest()
		{
			var lexer = new TemplateLexer(_templateString);
			var token = lexer.Next();

			do
			{
				var text = String.Format("[{0}, {1}] {2} {3}", token.Line.ToString("D3"), token.Column.ToString("D3"), token.Kind.ToString().PadLeft(10, ' '), token.Text.Replace("\r\n", "\\r\\n"));

				Trace.WriteLine(text);

				token = lexer.Next();
			}
			while (token.Kind != TokenKind.EOF);
		}
	}
}
