﻿// ===============================================================================
//  解决方案：置换型模板引擎
//  项目名称：ReplacementTemplateEngine
//  文件名称：TokenParser.cs
//  代码作者：网鸟老兵团(Me.YMind)
//  创建时间：2012-04-03 16:01
// ===============================================================================
//  Copyright © Htmlbird.Net. All rights reserved .
//  官方网站：http://www.htmlbird.net/
//  技术论坛：http://bbs.htmlbird.net/
//  版权所有：网鸟IT技术论坛 彦铭工作室
// ===============================================================================

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Text;
using NUnit.Framework;

namespace ReplacementTemplateEngine.bak1
{
	[TestFixture]
	public sealed class TokenParser
	{
		// 这俩还需要解释？？
		private const char _LABEL_OPEN_CHAR = '{';
		private const char _LABEL_CLOSE_CHAR = '}';

		/// <summary>
		/// 表示 Token 顺序集合（Token流）。
		/// </summary>
		private readonly List<string> _tokens = new List<string>();

		// 为有限状态机定义一个寄存器
		// 注意：有限状态机的理解在物理层的电路上和在编程概念上是相通的
		private readonly StringBuilder _temp = new StringBuilder();

		/// <summary>
		/// 表示当前状态。
		/// </summary>
		private ParserMode _currentMode;

		/// <summary>
		/// 表示上一状态。
		/// </summary>
		/// <remarks>
		/// 如果状态多余两个的话，我们总不能再定义一个"_last_last_Mode"吧！
		/// 在状态有多个的时候，需要使用 <see cref="Stack{T}"/> 来保存历史
		/// 状态，这个我们将在解释型模版引擎中用到。
		/// </remarks>
		private ParserMode _lastMode;

		/// <summary>
		/// 进入模式。
		/// </summary>
		/// <param name="mode"><see cref="ParserMode"/> 枚举值之一。</param>
		private void _EnterMode(ParserMode mode)
		{
			// 当状态改变的时候应当保存之前已处理的寄存器中的内容
			if (this._temp.Length > 0)
			{
				this._tokens.Add(this._temp.ToString());

				this._temp.Clear();
			}

			this._lastMode = this._currentMode;
			this._currentMode = mode;
		}

		/// <summary>
		/// 离开模式。
		/// </summary>
		private void _LeaveMode()
		{
			// 当状态改变的时候应当保存之前已处理的寄存器中的内容
			// 当状态超过2个的时候，实际上这里的代码应该是不一样的
			// 虽然现在我们只需要考虑两种状态，但为了更加直观的演示，我特意在这里又写了一遍
			if (this._temp.Length > 0)
			{
				this._tokens.Add(this._temp.ToString());
				this._temp.Clear();
			}

			// 因为只有两个状态，因此
			this._currentMode = this._lastMode;
		}

		public void ParseTemplate(string template)
		{
			if (String.IsNullOrEmpty(template)) return;

			var templateLength = template.Length;

			// 为了模拟光标的定位移动，我们在这里采用for而不是foreach
			// 在本例中用for还是foreach都无关紧要
			// 以后我们还会讨论更加复杂的情况，到时候就需要用到while(bool)了！
			for (var index = 0; index < templateLength; index++)
			{
				var c = template[index];

				switch (c)
				{
					case _LABEL_OPEN_CHAR:
						this._EnterMode(ParserMode.EnterLabel);

						// 将当前字符压入寄存器，同样的代码在三个分支都出现了
						// 请留意这行代码出现的时机
						// 在不同的状态下可能出现不同的处理过程
						this._temp.Append(c);
						break;

					case _LABEL_CLOSE_CHAR:
						// 将当前字符压入寄存器
						this._temp.Append(c);
						// 因为本例只有两个状态，因此这里相当于：_EnterMode(ParserMode.LeaveLabel)
						// 但是为了区别一下，我还是定义了两个方法来实现
						this._LeaveMode();
						break;

					default:
						// 将当前字符压入寄存器
						this._temp.Append(c);
						break;
				}
			}

			// 到达结尾的时候也需要处理寄存器中的内容
			// 这就是之前提到的硬编码解决开始和结束两个状态
			if (this._temp.Length <= 0) return;

			this._tokens.Add(this._temp.ToString());
			this._temp.Clear();
		}

		[Test]
		public void Test1()
		{
			var s = new StringBuilder();

			foreach (var c in TestObjects.TEMPLATE_STRING)
			{
				// 这里我们用回车换行符将输出隔开
				s.AppendLine(c.ToString(CultureInfo.InvariantCulture));
			}

			Trace.WriteLine(s.ToString());
		}

		[Test]
		public void Test2()
		{
			this.ParseTemplate(TestObjects.TEMPLATE_STRING);

			foreach (var token in this._tokens)
			{
				// 输出看看
				Trace.WriteLine(token);
			}
		}

		[Test]
		public void Test3()
		{
			this.ParseTemplate(TestObjects.TEMPLATE_STRING);

			foreach (var newsItem in TestObjects.NewsItems)
			{
				foreach (var token in this._tokens)
				{
					switch (token)
					{
						case "{url}":
							Trace.Write(newsItem.Key);
							break;

						case "{title}":
							Trace.Write(newsItem.Value);
							break;

						default:
							Trace.Write(token);
							break;
					}
				}

				Trace.WriteLine(String.Empty);
			}
		}
	}
}
